// 
// IBAction.cs
//  
// Author:
//       Michael Hutchinson <mhutchinson@novell.com>
// 
// Copyright (c) 2011 Novell, Inc.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace MonoDevelop.MacDev.ObjCIntegration
{
	public class IBAction
	{
		public IBAction (string objcName, string cliName)
		{
			this.ObjCName = objcName;
			this.CliName = cliName;
			this.Parameters = new List<IBActionParameter> ();
		}
		
		public string ObjCName { get; private set; }
		public string CliName { get; internal set; }
		public List<IBActionParameter> Parameters { get; private set; }
		public bool IsDesigner { get; internal set; }
		
		public string GetObjcFullName ()
		{
			if (Parameters == null || Parameters.Count == 0)
				return ObjCName;
			
			var sb = new System.Text.StringBuilder ();
			sb.Append (ObjCName);
			foreach (var p in Parameters) {
				if (p.Label != null)
					sb.Append (p.Label);
				sb.Append (":");
			}
			return sb.ToString ();
		}
	}
}
