// 
// IExecutionCommandEditor.cs
//  
// Author:
//       Lluis Sanchez Gual <lluis@novell.com>
// 
// Copyright (c) 2009 Novell, Inc (http://www.novell.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using MonoDevelop.Core.Execution;


namespace MonoDevelop.Ide.Execution
{
	/// <summary>
	/// This interface can be used to allow the user to customize a command.
	/// A customizer can for example set environment variables or modify the
	/// arguments of a command.
	/// </summary>
	public interface IExecutionCommandCustomizer
	{
		/// <summary>
		/// Must return true if the customizer can customize the provided command
		/// </summary>
		bool CanCustomize (ExecutionCommand cmd);
		
		/// <summary>
		/// Customizes the command with the provided data.
		/// </summary>
		void Customize (ExecutionCommand cmd, object data);
		
		/// <summary>
		/// Creates the editor to be used to enter the customization parameters
		/// </summary>
		IExecutionConfigurationEditor CreateEditor ();
	}
}
